const canvas = document.getElementById("canvas");
const gl=canvas.getContext("2d");
const canvas_width=canvas.width;
const canvas_height=canvas.height;

var objects=[];
const gravity=6;

//game variables
var player=null;
const player_width=50;
const player_height=50;
const player_tile_size=[16,16];
const playerAnimationCount={
    idle:6,
    run:8
}
const jumpHeight=150;
var animationFrame=0;

//settings for the game platforms
const platform_height=50;
const  platform_width=100;
//settings for the obstactles
const obstactleSpeed=15;
const obstactleSpawnTime=[2000,10000] //min and max
const obstactleHeight=40;
const obstacleWidth=40;
var prev_obstacle=null;
//image assets
const assets={
    background:"bg.jpg",
    platform:"platform.png",
    run:"run.png",
    idle:"idle.png",
    obstacle:"obstacle.png"

}
//lets add event to capture input from the player to jump
document.addEventListener("keyup",function(event){
 
    //lets the key w 
    if(event.key=="w"){
        jump();
        
    }
})
//function to do player jump
function jump(){
    if(is_grounded==true && init==true){
        player.y-=jumpHeight;
    }
}
//refresh rate for the game
const refresh_rate=100;
//game initation variables and checks
var all_assets_loaded=false;
var asset_count=0;
var game_started=false;
var is_game_over=false;
var init=false;
var is_grounded=true;

var score=0;
var score_timeout=1000;
//physics time
var dt=0.25;
var time_lapse=0;
//function to update the score
function updateScore(){
    if(is_game_over==false  && init==true && game_started==true){
       
        
        score+=1;
        setTimeout(updateScore,score_timeout);
    }
}
//function to draw the score on the screen
function drawScore(){
    var message="000000"+score;
    gl.font="32px Arial";
    var size=gl.measureText(message);
    gl.fillStyle='#fff'
    gl.fillText(message,canvas_width-size.width,32+10);
}
//function load assets
function load(){
    var keys=Object.keys(assets);
    asset_count=keys.length;
    for(i in keys){
        var key=keys[i];
        var src=assets[key];
        var img=new Image();
        img.src=src;
       
        img.onload=function(){
            asset_count-=1;
            
            if(asset_count==0){
                all_assets_loaded=true;
               
            }
        }
        assets[key]=img;
    }
}
//sprite class for drawing objects
class sprite{
    constructor(image,x,y,width,height,dx=0,dy=0,dwith=null,dheight=null){
        this.image=image;
        this.x=x;
        this.y=y;
        this.width=width;
        this.height=height;
        this.dx=dx;
        this.dy=dy;
        this.dwidth=dwith;
        this.dheight=dheight;
        this.type="sprite";
        objects.push(this);
    }
    reset(image,x,y,width,height,dx=0,dy=0,dwith=null,dheight=nul){
        this.image=image;
        this.x=x;
        this.y=y;
        this.width=width;
        this.height=height;
        this.dx=dx;
        this.dy=dy;
        this.dwidth=dwith;
        this.dheight=dheight;
    }
    bounds(){
        //for the obstacle lets add some offset to the bounds
        if(this.type=="obstacle"){
            var offset=[20,10];
            return {
                x:this.x,
                y:this.y,
                width:this.width-offset[0],
                height:this.height-offset[1]
            };

        }
        return {
            x:this.x,
            y:this.y,
            width:this.width,
            height:this.height
        };
    }
    update(){
       
        if(this.dwidth==null || this.dheight==null){
            gl.drawImage(this.image,this.x,this.y,this.width,this.height);
        
        }
        else{
            gl.drawImage(this.image,this.dx,this.dy,this.dwidth,this.dheight,this.x,this.y,this.width,this.height);
        }
        
    }

}
//lets add some obstactles to the game
function addObstacles(){
    var platform_y=canvas_height-platform_height;
    var offset=50;
    var obstacle=new sprite(assets.obstacle,canvas_width+offset,platform_y-obstactleHeight,obstacleWidth,obstactleHeight);
    //lets ensure that the obstactles are not spawned too close, lets leave a min distance of 5 times the player width
    if(prev_obstacle!=null){
        var separation = obstacle.x-prev_obstacle.x;
        if(separation<5*player_width){
            obstacle.x=prev_obstacle.x+5*player_width;
        }
    }
    
    obstacle.type="obstacle";
    prev_obstacle=obstacle;
    var random_time=Math.random()*(obstactleSpawnTime[1]-obstactleSpawnTime[0])+obstactleSpawnTime[1];
    setTimeout(addObstacles,random_time);
}
//function to check collision of obstacles with player
function checkCollision(rect1,rect2){
    if (rect1.x < rect2.x + rect2.width) {
        if (rect1.x + rect1.width > rect2.x) {
            if (rect1.y < rect2.y + rect2.height) {
                if (rect1.y + rect1.height > rect2.y) {
                    return true;
                }
            }
        }
    }
    return false;
}
//function to play the game
function Play(){
    var button=document.getElementById("button");
    //lets check for quit event
    if(button.innerHTML=='Quit'){
            objects.splice(0,objects.length);
            score=0;
            game_started=false;
            is_game_over=false;
            button.innerHTML="Play";
            //lets set the focus on the canvas
            canvas.focus();
    }
    else{
    //lets check for play events
    if(game_started==false){
            game_started=true;
            is_game_over=false;
            init=false;
            button.innerHTML="Quit";
            //lets set the focus on the canvas
            canvas.focus();
    }
    else{
        if(is_game_over==true){
             //lets set the game over button
             init=false;
             is_game_over=false;
             button.innerHTML="Quit";
             //lets set the focus on the canvas
             canvas.focus();
        }
    }
}

}

//function to clear the screen
function clear(){
    gl.beginPath();
    gl.fillStyle="#000";
    gl.fillRect(0,0,canvas_width,canvas_height);
    gl.closePath();
}

//function to update every frame of the game
function update(){

    
    //check if the game has started
    if(all_assets_loaded==true && game_started==true && is_game_over==false){
        clear()
        
        //lets initialize the game
        if(init==false){
            //lets clear the objects array
            objects.splice(0,objects.length);
            //adding walk platform
            var platform_count=parseInt(canvas_width/platform_width)+5;
            var index=0;
            
            while(index<platform_count){
               
                var platform=new sprite(assets.platform,index*platform_width,canvas_height-platform_height,platform_width,platform_height);
                platform.type="platform";
                index+=1;
            }
            //add player to the game
            var platform_y=canvas_height-platform_height;
            player=new sprite(assets.idle,canvas_width/2-platform_width/2,platform_y-player_height,player_width,player_height,0,0,player_tile_size[0],player_tile_size[1]);
            //lets start adding the obstacles
            var random_time=Math.random()*(obstactleSpawnTime[1]-obstactleSpawnTime[0])+obstactleSpawnTime[0];
            
            setTimeout(addObstacles,random_time);
            
            init=true;
            //lets the score update
            score=0;
            setTimeout(updateScore,score_timeout);
        }
        //draw game background
        gl.drawImage(assets.background,0,0,canvas_width,canvas_height);
        

         //adding gravity to the player
         var platform_y=canvas_height-platform_height;
         if((player.y+50)<platform_y){
            player.y+=gravity*time_lapse;
            is_grounded=false;
            player.reset(assets.idle,player.x,player.y,player_width,player_height,0,0,player_tile_size[0],player_tile_size[1]);
            time_lapse+=dt;
        }
        else{
            time_lapse=0;
            player.y=platform_y-player_height;
            is_grounded=true;
            //lets animation the player
            player.reset(assets.run,player.x,player.y,player_width,player_height,animationFrame*player_tile_size[0],0,player_tile_size[0],player_tile_size[1]);
            animationFrame+=1;
            
            if(animationFrame>=playerAnimationCount.run-1){
                animationFrame=0;
            }
        }
        
        //lets draw the score
        drawScore();
        //update objects on the screen
        for(i in objects){
            var obj=objects[i];
            if(obj==null){
                continue;
            }
            //lets make the obstacles move
            if(obj.type=="obstacle"){
                if(obj.x>0-obstacleWidth){
                    obj.x-=obstactleSpeed;
                }
                else{
                    //remove the obstacle from the update array and replace with a null value
                    alert("null created")
                    objects[i]=null;
                }
                //check if obstacle hit player
                if(checkCollision(player.bounds(),obj.bounds())==true){
                        is_game_over=true;
                        //lets show some game over text
                        var message="Game Over";
                        gl.font="32px Arial";
                        var size=gl.measureText(message)
                        gl.fillStyle="red";
                        gl.strokeStyle="yellow";
                        
                        gl.fillText(message,canvas_width/2-size.width/2,canvas_height/2-32/2);
                        gl.strokeText(message,canvas_width/2-size.width/2,canvas_height/2-32/2);
                        //lets set the game over button
                        var button=document.getElementById("button");
                        button.innerHTML="Play Again"
                  
                        
                }
            }
            //lets move the platforms
            if(obj.type=="platform"){
                obj.x-=obstactleSpeed;
                if(obj.x+platform_width<=0){
                    
                            
                    var platform_count=parseInt(canvas_width/platform_width);
                    obj.x=platform_count*platform_width-obstactleSpeed;
                    
                   
                }
             
              
            }
            obj.update();
        }
        //finally lets remove any null objects from the update
        objects = objects.filter(element => element !== null);
    }

    else{
        if(is_game_over==false){
        clear()
        //draw game background
        gl.drawImage(assets.background,0,0,canvas_width,canvas_height);
       //adding walk platform
       var platform_count=parseInt(canvas_width/platform_width)+5;
       var index=0;
       
       while(index<platform_count){
          
         
           gl.drawImage(assets.platform,index*platform_width,canvas_height-platform_height,platform_width,platform_height);
           index+=1;
       }
       //lets also draw the score
       drawScore();
       //lets idle animate the player
       var platform_y=canvas_height-platform_height;

       gl.drawImage(assets.idle,animationFrame*player_tile_size[0],0,player_tile_size[0],player_tile_size[1],canvas_width/2-platform_width/2,platform_y-player_height,player_width,player_height);
       animationFrame+=1;
       
       if(animationFrame>=playerAnimationCount.idle-1){
           animationFrame=0;
       }
   //lets add game title
   var message="First Javascript Game";
   gl.font="30px Arial";
   var size=gl.measureText(message)
   gl.fillStyle="blue";
   gl.strokeStyle="yellow";
                   
   gl.fillText(message,canvas_width/2-size.width/2,canvas_height/2-32/2);
   gl.strokeText(message,canvas_width/2-size.width/2,canvas_height/2-32/2);
        }
     
        

    }
    


    setTimeout(update,refresh_rate)
}
load();
update();
